import 'dart:collection';
import 'package:flutter/foundation.dart';

import '../src/bdlive_flutter_viewer_impl.dart';

class TTSDKConfiguration {
  final String appID; // appID，非空，必填
  final String liveLicenseFilePath; // 非空，必填 ，live license本地文件路径
  final String vodLicenseFilePath; // 非空，必填 ，vod license本地文件路径
  String? bundleID; // IOS App的唯一标识
  String? appName; // app name
  String? channel; // App 的渠道名称
  String? appVersion; // App 的版本号。合法版本号应包含 2 个或以上的分隔符，例如 1.3.2

  TTSDKConfiguration({
    required this.appID,
    required this.liveLicenseFilePath,
    required this.vodLicenseFilePath,
    this.bundleID,
    this.appName,
    this.channel,
    this.appVersion,
  });

  Map<String, dynamic> toMap() {
    HashMap<String, dynamic> dic = HashMap();
    dic['appID'] = appID;
    dic['liveLicenseFilePath'] = liveLicenseFilePath;
    dic['vodLicenseFilePath'] = vodLicenseFilePath;
    if (bundleID != null) {
      dic['bundleID'] = bundleID;
    }
    if (appName != null) {
      dic['appName'] = appName;
    }
    if (channel != null) {
      dic['channel'] = channel;
    }
    if (appVersion != null) {
      dic['appVersion'] = appVersion;
    }
    return dic;
  }
}

enum BdlViewerAuthMode {
  // 公开模式，观众以游客身份进入直播间，在点击评论输入框或点赞评论等需要用户信息的场景下，需要先完成游客注册（即输入昵称）。
  public,
  // 自定义模式，观众在进入直播间时使用的是在您 App 内的用户信息，因此可以直接发送评论、点赞评论等。
  custom,
}

enum BdlViewerOpenUrlType {
  // 页头广告
  pageHeaderAd,
  // 页中广告
  pageAd,
  // 浮标广告
  floatingAd,
  // 商品卡片，包括商品列表中的商品和浮标商品
  cardAd,
}

class BdlActivityConfig {
  //直播间id
  final int activityId;
  //直播间进房token
  final String token;
  //直播间鉴权模式，详见 BdlViewerAuthMode
  final BdlViewerAuthMode authMode;
  //要进入横版还是竖版直播间，true：竖版；false：横版
  final bool isPortrait;

  BdlActivityConfig({
    required this.activityId,
    required this.token,
    required this.authMode,
    required this.isPortrait,
  });

  Map<String, dynamic> toMap() {
    HashMap<String, dynamic> dic = HashMap();
    dic['activityId'] = activityId;
    dic['token'] = token;
    dic['authMode'] = authMode.index + 1;
    dic['isPortrait'] = isPortrait;
    return dic;
  }

  factory BdlActivityConfig.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('BdlActivityConfig fromMap = $map');
    int activityId = map['activityId'] ?? 0;
    String token = map['token'] ?? '';
    int authMode = map['authMode'] ?? 0;
    bool isPortrait = map['isPortrait'] ?? false;
    return BdlActivityConfig(
        activityId: activityId,
        token: token,
        authMode: BdlViewerAuthMode.values[authMode - 1],
        isPortrait: isPortrait);
  }

  @override
  toString() {
    return 'BdlActivityConfig {activityId: $activityId, token: $token, authMode: $authMode, isPortrait: $isPortrait}';
  }
}

class BdlLiveRoomStatusListener {
  /**
   * 直播间页面进入前台时的回调
   * @param activity:进入直播间时的配置，详见 BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onLiveRoomActivityResume;

  /**
   * 直播间页面进入后台时的回调
   * @param activity:进入直播间时的配置，详见 BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onLiveRoomActivityPause;

  /**
   * 直播间小窗即将显示时的回调
   * @param activity:进入直播间时的配置，详见 BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onFloatViewWillAppear;

  /**
   * 直播间小窗即将消失时的回调
   * @param activity:进入直播间时的配置，详见 BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onFloatViewWillClose;

  /**
   * 直播间中的广告/商品卡片被点击时的回调，客户可以监听到这个回调做自定义跳转
   * @param url:广告/商品卡片对应的跳转链接
   * @param type:广告/商品卡片的类型，详见 BdlViewerOpenUrlType
   * @param enableFloating:用户在企业直播控制台是否设置为观看页浮层展示内容。
      true：在观看页浮层展示内容。
      false：在新页面展示内容。
   */
  void Function(String url, BdlViewerOpenUrlType type, bool enableFloating)? onOpenUrlCallback;

  BdlLiveRoomStatusListener({
    this.onLiveRoomActivityResume,
    this.onLiveRoomActivityPause,
    this.onFloatViewWillAppear,
    this.onFloatViewWillClose,
    this.onOpenUrlCallback,
  });
}

abstract class BdliveViewerApi {

  static BdliveViewerApi _instance = BdliveFlutterViewerImpl();
  static BdliveViewerApi get instance => _instance;
  static set instance(BdliveViewerApi instance) {
    _instance = instance;
  }

  // 使用给定的配置初始化SDK，启动app时调用
  static Future<void> startWithConfiguration(TTSDKConfiguration config) =>
      BdliveFlutterViewerImpl.startWithConfiguration(config);

  /**
   * 进入完整直播间页面
   * @param activity:进入直播间时的配置，详见 BdlActivityConfig
   */
  Future<bool> enterLiveRoom(BdlActivityConfig activity);


  /**
   * 离开完整直播间页面，如果此时正在显示直播间小窗，则直播间小窗也会被销毁
   */
  Future<void> leaveLiveRoom();

  /**
   * 模拟手动点击完整直播间页面中的退出直播间图标，之后的行为和观众正常点击退出直播间图标的行为一致。
   */
  Future<void> manualClickLiveRoomExitBtn();

  /**
   * 给完整直播间页面设置监听器，用于监听完整直播间页面的事件
   * @param listener: 完整直播间页面设置监听器,详见 BdlLiveRoomStatusListener
   */
  Future<void> setLiveRoomStatusListener(BdlLiveRoomStatusListener listener);

  /**
   * 配合BdlLiveRoomStatusListener中的onOpenUrlCallback回调使用，
   * 客户可以调用这个API使用SDK的一些默认跳转能力

   * @param url:广告/商品卡片对应的跳转链接
   * @param type:广告/商品卡片的类型，详见 BdlViewerOpenUrlType
   * @param enableFloating:是否允许在观看页浮层展示内容。
      true：在观看页浮层展示内容，此时会在完整直播页中弹出webview弹框展示Url。
      false：在新页面展示内容，此时会通过app的deeplink机制寻找符合条件的页面打开
   */
  Future<void> openUrl(String url, BdlViewerOpenUrlType type, bool enableFloating);


}
